#!/bin/bash
# 包含GMT自带脚本文件 其中包含了一些有用的功能 比如获取网格文件的范围
. gmt_shell_functions.sh
# 包含dispOption脚本
. dispOptions
#GMT显示平面数据脚本，输入文件为网格文件，没有包含网格化语句因为网格化过程中的情况多样化，建议在其他脚本中个别添加再调用此脚本
# 初始化参数
data='null'
gridData='null'
unit='m'
color='rainbow'
range='null'
overwriteRange='null'
overwrite=0
labels=("longitude (degree)" "latitude (degree)")
plotgrad=0
clon=0
polyfile='null'
# 从命令行获取参数
while getopts "hi:r:u:c:a:G:l:p:g" arg
do
	case $arg in
		h)
			dispTitle "${0##*/}" "simple global wide data mapping using GMT script. The template accepts a grid (.nc .grid) file as input and outputs a .png and a .eps file. \
For futher explanations, please look for GMT's manuscripts."
			dispAuthorInfo "Yi Zhang (zhangyi.cugwuhan@gmail.com)"
			dispUsage "${0##*/} -i<grid-data> [-r<xmin>/<xmax>/<ymin>/<ymax>] [-l<lon>] [-u<unit>] [-c<cpt-file>] [-a<x-label>,<y-label>] [-g] [-G<grad-data>] [-p<polygon-file>]"
			dispOptionShort "-i" "input grid file"
			dispOptionShort "-r" "data range. The template will ditect the input data range automaticly, the use of this option will overwrite the range."
			dispOptionShort "-l" "central longitude value of the plot, the default value is 0"
			dispOptionShort "-u" "data unit. The default is meter. use 'km+Uk' to append '+Uk' option to the psscale command"
			dispOptionShort "-c" "color cpt. The default is rainbow."
			dispOptionShort "-a" "axis labels. The deafults are x (m) and y (m)."
			dispOptionShort "-g" "use grdgradient. The default is false."
			dispOptionShort "-G" "provide a different input grid data for the use of grdgradient. This should be used with the '-g' option at the same time"
			dispOptionShort "-p" "plot polygons via a file."
			exit 0;;
		i)
			data=$OPTARG;;
		u)
			unit=$OPTARG;;
		c)
			color=$OPTARG;;
		r)
			overwrite=1
			overwriteRange=$OPTARG;;
		l)
			clon=$OPTARG;;
		a)
			labels=(${OPTARG//,/ });;
		g)
			plotgrad=1;;
		G)
			gridData=$OPTARG;;
		p)
			polyfile=$OPTARG;;
		?)
			printf "error: unknow argument\nuse -h option to see help information\n"
			exit 1;;
	esac
done
# 进行必要的参数检查
if [[ $data == "null" ]]; then
	printf "error: no input file name\nuse -h option to see help information\n"
	exit 1
else
	# 初始化临时文件名
	cptfile=user.cpt
	psfile=${data%.*}.ps
	jpgfile=${data%.*}.png
	# 获取网格范围
	if [[ $overwrite == 1 ]]; then
		range=${overwriteRange}
	else
		range=$(gmt_get_gridregion ${data})
	fi
	# 设置绘图参数
	gmt gmtset \
	FONT_ANNOT_PRIMARY=7p,Times-Roman,black \
	MAP_FRAME_PEN=thinnest,black \
	MAP_TICK_LENGTH_PRIMARY=1.5p/1p \
	MAP_TICK_PEN_PRIMARY=thinnest,black \
	MAP_TITLE_OFFSET=6.5p \
	MAP_GRID_CROSS_SIZE_PRIMARY=2p \
	FONT_LABEL=7p,Times-Roman,black \
	MAP_FRAME_AXES=WESnZ \
	MAP_LABEL_OFFSET=2.5p

	gmt grd2cpt ${data} -C${color} -Z -D > $cptfile
	if [[ $plotgrad == 1 ]]; then
		gradfile=${data%.*}Grad.nc
		if [[ $gridData == 'null' ]]; then
			gmt grdgradient ${data} -G${gradfile} -Nt -A0/45
		else
			gmt grdgradient ${gridData} -G${gradfile} -Nt -A0/45
		fi
		gmt grdimage ${data} -R${range} -C${cptfile} -I${gradfile} -Bxag+l"${labels[0]}" -Bpya40g40+l"${labels[1]}" -JN${clon}/2i -K -P > $psfile
	else
		gmt grdimage ${data} -R${range} -C${cptfile} -Bxag+l"${labels[0]}" -Bpya40g40+l"${labels[1]}" -JN${clon}/2i -K -P > $psfile
	fi
	if [[ $polyfile != 'null' ]]; then
		gmt psxy $polyfile -JN${clon}/2i -R${range} -L -K -O >> $psfile
	fi
	#-C${cptfile}+Uk 使用km（色标单位除1000
	#如果unit等于km则在cptfile后面添加+Uk
	if [[ ${unit} == 'km+Uk' ]]; then
		gmt psscale -Dx0.4i/-0.3i+w1.2i/0.05i+h -C${cptfile}+Uk -Bxa -By+l"km" -O >> $psfile
	else
		gmt psscale -Dx0.4i/-0.3i+w1.2i/0.05i+h -C${cptfile} -Bxa -By+l${unit} -O >> $psfile
	fi
	gmt psconvert $psfile -A -TEG -E300
	# 删除临时文件 使用linux终端rm命令
	rm $cptfile $psfile gmt.history gmt.conf
	if [[ $plotgrad == 1 ]]; then
		rm $gradfile
	fi
	# 在终端显示图像 此命令需要imgcat.sh脚本和iTerm终端
	#imgcat $jpgfile
	# 打开图片文件 此命令使用MacOS终端open命令
	open $jpgfile
fi