#!/bin/bash
# 包含GMT自带脚本文件 其中包含了一些有用的功能 比如获取网格文件的范围
. gmt_shell_functions.sh
# 初始化参数
data='null'
unit='m'
color='rainbow'
range='null'
overwriteRange='null'
overwrite=0
labels=("x (m)" "y (m)")
valuerange="0/1000/10"
noshow=0

# 从命令行获取参数
while getopts "hqi:r:u:c:a:d:" arg
do
	case $arg in
		h)
			printf "simple table file mapping using GMT script under x-y plane with colored dots. The template accepts a table file as input and outputs a .png and a .eps file. \
For futher explanations, please look for GMT's manuscripts.\nAuthor: Yi Zhang (zhangyi.cugwuhan@gmail.com)\n" 
			printf "usage: ${0##*/} -i<table-data> [-r<xmin>/<xmax>/<ymin>/<ymax>] [-u<unit>] [-c<cpt-file>] [-d<vmin>/<vmax>/<dv>] [-a<x-label>,<y-label>]\n"
			printf "%s\t%s\n" "-i" "input grid file"
			printf "%s\t%s\n" "-r" "data range. The template will ditect the input data range automaticly, the use of this option will overwrite the range."
			printf "%s\t%s\n" "-d" "data's value extension. The default is 0/1000/10. this option is used for makeing cpt file."
			printf "%s\t%s\n" "-u" "data unit. The default is meter."
			printf "%s\t%s\n" "-c" "color cpt. The default is rainbow."
			printf "%s\t%s\n" "-a" "axis labels. The deafults are x (m) and y (m)."
			printf "%s\t%s\n" "-q" "do not show the output figure."
			exit 0;;
		i)
			data=$OPTARG;;
		u)
			unit=$OPTARG;;
		c)
			color=$OPTARG;;
		r)
			overwrite=1
			overwriteRange=$OPTARG;;
		a)
			labels=(${OPTARG//,/ });;
		d)
			valuerange=$OPTARG;;
		q)
			noshow=1;;
		?)
			printf "error: unknow argument\nuse -h option to see help information\n"
			exit 1;;
	esac
done

# 进行必要的参数检查
if [[ $data == "null" ]]; then
	printf "error: no input file name\nuse -h option to see help information\n"
	exit 1
else
	# 初始化临时文件名
	cptfile=user.cpt
	psfile=${data%.*}.ps
	jpgfile=${data%.*}.png
	# 获取网格范围
	if [[ $overwrite == 1 ]]; then
		range=${overwriteRange}
	else
		range=$(gmt_get_region ${data})
	fi
	# 设置绘图参数
	gmt gmtset \
	FONT_ANNOT_PRIMARY=7p,Times-Roman,black \
	MAP_FRAME_PEN=thin,black \
	MAP_TICK_LENGTH_PRIMARY=2p/1p \
	MAP_TITLE_OFFSET=7p \
	MAP_GRID_CROSS_SIZE_PRIMARY=4p \
	FONT_LABEL=7p,Times-Roman,black \
	MAP_FRAME_AXES=WeSnZ \
	MAP_LABEL_OFFSET=4p

	gmt makecpt -C${color} -T${valuerange} > $cptfile
	gmt psxy ${data} -Bxag+l"${labels[0]}" -Byag+l"${labels[1]}" -JX5i -R${range} -C${cptfile} -Sc0.05c -Gblack -P -K -Y6i > $psfile
	gmt psscale -Dx0.5i/-0.6i+w4i/0.15i+h -C${cptfile} -Bxag -By+l${unit} -O >> $psfile 
	gmt psconvert $psfile -A -TEG -E300
	# 删除临时文件 使用linux终端rm命令
	rm $cptfile $psfile gmt.history gmt.conf
	# 在终端显示图像 此命令需要imgcat.sh脚本和iTerm终端
	#imgcat $jpgfile
	# 打开图片文件 此命令使用MacOS终端open命令
	if [[ ${noshow} == 0 ]]; then
		open $jpgfile
	fi
fi